/*
 * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* $Id: kafs_locl.h,v 1.15 1999/12/02 16:58:40 joda Exp $ */

#ifndef __KRBAFS_LOCL_H__
#define __KRBAFS_LOCL_H__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <setjmp.h>
#include <errno.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifndef HAVE_U_INT32_T
typedef unsigned long u_int32_t;
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_CRYPT_H
#include <crypt.h>
#endif
#if defined(HAVE_SYS_IOCTL_H) && SunOS != 40
#include <sys/ioctl.h>
#endif
#ifdef HAVE_SYS_FILIO_H
#include <sys/filio.h>
#endif

#ifdef HAVE_SYS_SYSCALL_H
#include <sys/syscall.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_NETINET_IN6_H
#include <netinet/in6.h>
#endif
#ifdef HAVE_NETINET6_IN6_H
#include <netinet6/in6.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_ARPA_NAMESER_H
#include <arpa/nameser.h>
#endif
#ifdef HAVE_RESOLV_H
#include <resolv.h>
#endif

#ifdef HAVE_KRB5_H
#include <krb5.h>
#endif
#ifdef HAVE_KRB524_H
#include <krb524.h>
#endif
#ifdef HAVE_KRB_H
#include <krb.h>
#endif
#include <krbafs.h>

#define dns_free_data _kafs_dns_free_data
#define dns_lookup _kafs_dns_lookup
#define dns_srv_order _kafs_dns_srv_order
#define dns_string_to_type _kafs_dns_string_to_type
#define dns_type_to_string _kafs_dns_type_to_string

#include "resolve.h"

#include "afssysdefs.h"

#ifdef NEED_STRTOK_R_PROTO
#ifndef HAVE_STRTOK_R
#define strtok_r _kafs_strtok_r
char *_kafs_strtok_r(char *s1, const char *s2, char **lasts);
#endif
char *strtok_r(char *s1, const char *s2, char **lasts);
#endif

#ifndef HAVE_STRLCPY
#define strlcpy _kafs_strlcpy
size_t _kafs_strlcpy (char *dst, const char *src, size_t dst_sz);
#endif

#ifndef HAVE_AFS_STRING_TO_KEY
void afs_string_to_key(const char *pass, const char *cell, des_cblock *key);
#endif

#ifndef HAVE_KRB_TIME_TO_LIFE
int krb_time_to_life(unsigned long start, unsigned long end);
#endif

#ifndef HAVE_KRB_ATIME_TO_LIFE
int krb_atime_to_life(char *atime);
#endif

#ifndef HAVE_KRB_LIFE_TO_TIME
unsigned long krb_life_to_time(unsigned long start, int life);
#endif

#ifndef HAVE_KRB_LIFE_TO_ATIME
char *krb_life_to_atime(int life);
#endif

struct kafs_data;
typedef int (*afslog_uid_func_t)(struct kafs_data *,
				 const char *cell,
				 const char *realm_hint,
				 uid_t,
				 const char *homedir);

typedef int (*get_cred_func_t)(struct kafs_data*, const char*, const char*, 
				    const char*, CREDENTIALS*);

typedef char* (*get_realm_func_t)(struct kafs_data*, const char*);

typedef struct kafs_data {
    afslog_uid_func_t afslog_uid;
    get_cred_func_t get_cred;
    get_realm_func_t get_realm;
    void *data;
} kafs_data;

int _kafs_afslog_all_local_cells(kafs_data*, uid_t, const char*);

int _kafs_get_cred(kafs_data*, const char*, const char*, const char *, 
		  CREDENTIALS*);

int
_kafs_realm_of_cell(kafs_data *data, const char *cell, char **realm);

#ifdef _AIX
int aix_pioctl(char*, int, struct ViceIoctl*, int);
int aix_setpag(void);
#endif

#endif /* __KRBAFS_LOCL_H__ */
